import Component from '@glimmer/component';
import { tracked } from '@glimmer/tracking';
import { scaleTime, scaleLinear } from "d3-scale";

export default class PlotComponent extends Component {
  @tracked dimensions;
  @tracked elementWidth;
  @tracked elementHeight;

  get heightRatio() {
    return this.args.heightRatio || 0.5;
  }

  get height() {
    return (this.elementWidth ? this.elementWidth : 0) * this.heightRatio;
  }

  get useElementHeight() {
    return this.args.useElementHeight || false;
  }

  get transform() {
    return this.args.transform || "translate(0,0)";
  }

  get hasData() {
    return this.args.data && this.args.data.length > 0;
  }

  get isMultiDataset() {
    return this.hasData && this.args.data[0] && this.args.data[0].constructor === Array;
  }

  get isMultiRange() {
    return this.args.timeRange && this.args.timeRange[0].constructor === Array;
  }

  get matchMultiRangeXscale() {
    return this.args.matchMultiRangeXscale || true;
  }

  get scaleYBasedOnTotalDataset() {
    return this.args.scaleYBasedOnTotalDataset || false;
  }

  get yScale() {
    return this.yScaleRange.domain(this.yRange());
  }

  get showingYLabel() {
    return !this.args.isLoading && this.args.yLabel;
  }

  get showingXLabel() {
    return !this.args.isLoading && this.args.xLabel;
  }

  get xScale() {
    return this.isMultiRange
      ? this.sortedTimeRange.map(range => this.xScaleRange.domain(range))
      : this.xScaleRange.domain(this.sortedTimeRange);
  }

  sortDates(d1, d2) {
    return d1 - d2;
  }

  get groupDimensions() {
    return {
      height: this.dimensions ? this.dimensions.svg.clientHeight : 0,
      width: this.dimensions ? this.dimensions.svg.clientWidth : 0
    }
  }

  get sortedTimeRange() {
    return this.args.timeRange
      ? this.isMultiRange
        ? this.adjustRanges(this.args.timeRange.map(range => range.sort(this.sortDates)))
        : this.args.timeRange.sort(this.sortDates)
      : [new Date(), new Date()]
  }

  adjustRanges(ranges) {
    if(ranges && this.isMultiRange && this.matchMultiRangeXscale) {
      const rangeLengths = ranges.map(range => range[1] - range[0])
      const maxRange = Math.max(...rangeLengths)
      return ranges.map((range) => {
        const additionalScale = maxRange - (range[1] - range[0])
        return [ range[0], new Date(range[1].getTime() + additionalScale) ]
      })
    } else {
      return ranges;
    }
  }

  get xKey() {
    return this.args.xKey ? this.args.xKey : 'at';
  }

  get yKey() {
    return this.args.yKey ? this.args.yKey : 'value';
  }

  get xScaleRange() {
    return this.dimensions
    ? scaleTime().range([0, this.dimensions.width])
    : {}
  }

  get yScaleRange() {
    return this.dimensions
    ? scaleLinear().range([this.dimensions.height, 0])
      : {}
  }

  get filteredData() {
    if (!this.hasData) return []
    if (this.isMultiDataset && this.isMultiRange) {
      return this.sortedTimeRange.map((range, index) => this.filterData(this.args.data[index], range))
    } else if (this.isMultiDataset) {
      return this.args.data.map(dataset => this.filterData(dataset));
    } else if (this.isMultiRange) {
      return this.sortedTimeRange.map(range => this.filterData(this.args.data, range))
    } else {
      return this.filterData(this.args.data);
    }
  } 

  filterData(dataset, timeRange) {
    const range = timeRange ? timeRange : this.sortedTimeRange
    return dataset.filter((dataPoint) => {
      return dataPoint[this.xKey].getTime() > range[0].getTime() && dataPoint[this.xKey].getTime() < range[1].getTime();
    })
  }

  // Amount of space to buffer the min data (if set to zero, lowest datapoint will be on x axis)
  get dataBottomPadding() {
    return this.args.dataBottomPadding ? this.args.dataBottomPadding : 0
  }

  yRange() {
    if(!this.args.data) return [0,0]
    const y = this.yKey;
    const data = this.scaleYBasedOnTotalDataset ? this.args.data : this.filteredData;
    const dataset = this.isMultiDataset || (!this.scaleYBasedOnTotalDataset && this.isMultiRange) ? data.flat(1) : data
    const max = dataset.length > 0 ? Math.max(...dataset.map(d => d[y])) : 0;
    return [
      0,
      max * 1.05
    ]
  }

  resize = () => {
    const chart = this.dimensions.svg;
    this.elementWidth = chart.getBoundingClientRect().width;
    this.dimensions = { width: this.elementWidth, height: this.height, svg: chart };
    if(this.args.updateDimensions) this.args.updateDimensions(this.dimensions);
  }

  initializeChart = (el) => {
    const boundingRect = el.getBoundingClientRect() 
    this.elementWidth = boundingRect.width;
    this.elementHeight = boundingRect.height;
    this.dimensions = {width: this.elementWidth, height: this.useElementHeight ? this.elementHeight : this.height, svg: el};
    if(this.args.updateDimensions) this.args.updateDimensions( this.dimensions );
  }
}